@@ -26,7 +26,6 @@
 #include "gnutls_int.h"
 #include "gnutls_algorithms.h"
 #include "gnutls_errors.h"
-#include "gnutls_cert.h"
 #include <x509/common.h>
 
 
@@ -1572,7 +1571,7 @@ gnutls_cipher_suite_get_name (gnutls_kx_algorithm_t kx_algorithm,
  * @kx: output variable indicating key exchange algorithm, or %NULL.
  * @cipher: output variable indicating cipher, or %NULL.
  * @mac: output variable indicating MAC algorithm, or %NULL.
- * @version: output variable indicating TLS protocol version, or %NULL.
+ * @min_version: output variable indicating TLS protocol version, or %NULL.
  *
  * Get information about supported cipher suites.  Use the function
  * iteratively to get information about all supported cipher suites.
