@@ -28,11 +28,11 @@
 #include "gnutls_auth.h"
 #include "gnutls_auth.h"
 #include "gnutls_algorithms.h"
-#include "auth_cert.h"
-#include "auth_psk.h"
+#include <auth/cert.h>
+#include <auth/psk.h>
+#include <auth/anon.h>
 #include <gnutls_datum.h>
 
-#include "auth_anon.h"
 /* The functions here are used in order for authentication algorithms
  * to be able to retrieve the needed credentials eg public and private
  * key etc.
@@ -48,15 +48,15 @@ void
 gnutls_credentials_clear (gnutls_session_t session)
 {
   if (session->key && session->key->cred)
-    {				/* beginning of the list */
+    {                           /* beginning of the list */
       auth_cred_st *ccred, *ncred;
       ccred = session->key->cred;
       while (ccred != NULL)
-	{
-	  ncred = ccred->next;
-	  gnutls_free (ccred);
-	  ccred = ncred;
-	}
+        {
+          ncred = ccred->next;
+          gnutls_free (ccred);
+          ccred = ncred;
+        }
       session->key->cred = NULL;
     }
 }
@@ -97,17 +97,17 @@ gnutls_credentials_clear (gnutls_session_t session)
  **/
 int
 gnutls_credentials_set (gnutls_session_t session,
-			gnutls_credentials_type_t type, void *cred)
+                        gnutls_credentials_type_t type, void *cred)
 {
   auth_cred_st *ccred = NULL, *pcred = NULL;
   int exists = 0;
 
   if (session->key->cred == NULL)
-    {				/* beginning of the list */
+    {                           /* beginning of the list */
 
       session->key->cred = gnutls_malloc (sizeof (auth_cred_st));
       if (session->key->cred == NULL)
-	return GNUTLS_E_MEMORY_ERROR;
+        return GNUTLS_E_MEMORY_ERROR;
 
       /* copy credentials locally */
       session->key->cred->credentials = cred;
@@ -119,36 +119,36 @@ gnutls_credentials_set (gnutls_session_t session,
     {
       ccred = session->key->cred;
       while (ccred != NULL)
-	{
-	  if (ccred->algorithm == type)
-	    {
-	      exists = 1;
-	      break;
-	    }
-	  pcred = ccred;
-	  ccred = ccred->next;
-	}
+        {
+          if (ccred->algorithm == type)
+            {
+              exists = 1;
+              break;
+            }
+          pcred = ccred;
+          ccred = ccred->next;
+        }
       /* After this, pcred is not null.
        */
 
       if (exists == 0)
-	{			/* new entry */
-	  pcred->next = gnutls_malloc (sizeof (auth_cred_st));
-	  if (pcred->next == NULL)
-	    return GNUTLS_E_MEMORY_ERROR;
+        {                       /* new entry */
+          pcred->next = gnutls_malloc (sizeof (auth_cred_st));
+          if (pcred->next == NULL)
+            return GNUTLS_E_MEMORY_ERROR;
 
-	  ccred = pcred->next;
+          ccred = pcred->next;
 
-	  /* copy credentials locally */
-	  ccred->credentials = cred;
+          /* copy credentials locally */
+          ccred->credentials = cred;
 
-	  ccred->next = NULL;
-	  ccred->algorithm = type;
-	}
+          ccred->next = NULL;
+          ccred->algorithm = type;
+        }
       else
-	{			/* modify existing entry */
-	  ccred->credentials = cred;
-	}
+        {                       /* modify existing entry */
+          ccred->credentials = cred;
+        }
     }
 
   return 0;
@@ -179,9 +179,9 @@ gnutls_auth_get_type (gnutls_session_t session)
 
   return
     _gnutls_map_kx_get_cred (_gnutls_cipher_suite_get_kx_algo
-			     (&session->
-			      security_parameters.current_cipher_suite),
-			     server);
+                             (&session->
+                              security_parameters.current_cipher_suite),
+                             server);
 }
 
 /**
@@ -200,8 +200,8 @@ gnutls_auth_server_get_type (gnutls_session_t session)
 {
   return
     _gnutls_map_kx_get_cred (_gnutls_cipher_suite_get_kx_algo
-			     (&session->
-			      security_parameters.current_cipher_suite), 1);
+                             (&session->
+                              security_parameters.current_cipher_suite), 1);
 }
 
 /**
@@ -220,8 +220,8 @@ gnutls_auth_client_get_type (gnutls_session_t session)
 {
   return
     _gnutls_map_kx_get_cred (_gnutls_cipher_suite_get_kx_algo
-			     (&session->
-			      security_parameters.current_cipher_suite), 0);
+                             (&session->
+                              security_parameters.current_cipher_suite), 0);
 }
 
 
@@ -231,12 +231,12 @@ gnutls_auth_client_get_type (gnutls_session_t session)
  */
 const void *
 _gnutls_get_kx_cred (gnutls_session_t session,
-		     gnutls_kx_algorithm_t algo, int *err)
+                     gnutls_kx_algorithm_t algo, int *err)
 {
   int server = session->security_parameters.entity == GNUTLS_SERVER ? 1 : 0;
 
   return _gnutls_get_cred (session->key,
-			   _gnutls_map_kx_get_cred (algo, server), err);
+                           _gnutls_map_kx_get_cred (algo, server), err);
 }
 
 const void *
@@ -253,9 +253,9 @@ _gnutls_get_cred (gnutls_key_st key, gnutls_credentials_type_t type, int *err)
   while (ccred != NULL)
     {
       if (ccred->algorithm == type)
-	{
-	  break;
-	}
+        {
+          break;
+        }
       ccred = ccred->next;
     }
   if (ccred == NULL)
@@ -315,47 +315,47 @@ _gnutls_free_auth_info (gnutls_session_t session)
       break;
     case GNUTLS_CRD_ANON:
       {
-	anon_auth_info_t info = _gnutls_get_auth_info (session);
+        anon_auth_info_t info = _gnutls_get_auth_info (session);
 
-	if (info == NULL)
-	  break;
+        if (info == NULL)
+          break;
 
-	dh_info = &info->dh;
-	_gnutls_free_dh_info (dh_info);
+        dh_info = &info->dh;
+        _gnutls_free_dh_info (dh_info);
       }
       break;
     case GNUTLS_CRD_PSK:
       {
-	psk_auth_info_t info = _gnutls_get_auth_info (session);
+        psk_auth_info_t info = _gnutls_get_auth_info (session);
 
-	if (info == NULL)
-	  break;
+        if (info == NULL)
+          break;
 
-	dh_info = &info->dh;
-	_gnutls_free_dh_info (dh_info);
+        dh_info = &info->dh;
+        _gnutls_free_dh_info (dh_info);
       }
       break;
     case GNUTLS_CRD_CERTIFICATE:
       {
-	unsigned int i;
-	cert_auth_info_t info = _gnutls_get_auth_info (session);
+        unsigned int i;
+        cert_auth_info_t info = _gnutls_get_auth_info (session);
 
-	if (info == NULL)
-	  break;
+        if (info == NULL)
+          break;
 
-	dh_info = &info->dh;
-	rsa_info = &info->rsa_export;
-	for (i = 0; i < info->ncerts; i++)
-	  {
-	    _gnutls_free_datum (&info->raw_certificate_list[i]);
-	  }
+        dh_info = &info->dh;
+        rsa_info = &info->rsa_export;
+        for (i = 0; i < info->ncerts; i++)
+          {
+            _gnutls_free_datum (&info->raw_certificate_list[i]);
+          }
 
-	gnutls_free (info->raw_certificate_list);
-	info->raw_certificate_list = NULL;
-	info->ncerts = 0;
+        gnutls_free (info->raw_certificate_list);
+        info->raw_certificate_list = NULL;
+        info->ncerts = 0;
 
-	_gnutls_free_dh_info (dh_info);
-	_gnutls_free_rsa_info (rsa_info);
+        _gnutls_free_dh_info (dh_info);
+        _gnutls_free_rsa_info (rsa_info);
       }
 
 
@@ -379,60 +379,60 @@ _gnutls_free_auth_info (gnutls_session_t session)
  */
 int
 _gnutls_auth_info_set (gnutls_session_t session,
-		       gnutls_credentials_type_t type, int size,
-		       int allow_change)
+                       gnutls_credentials_type_t type, int size,
+                       int allow_change)
 {
   if (session->key->auth_info == NULL)
     {
       session->key->auth_info = gnutls_calloc (1, size);
       if (session->key->auth_info == NULL)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_MEMORY_ERROR;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_MEMORY_ERROR;
+        }
       session->key->auth_info_type = type;
       session->key->auth_info_size = size;
     }
   else
     {
       if (allow_change == 0)
-	{
-	  /* If the credentials for the current authentication scheme,
-	   * are not the one we want to set, then it's an error.
-	   * This may happen if a rehandshake is performed an the
-	   * ciphersuite which is negotiated has different authentication
-	   * schema.
-	   */
-	  if (gnutls_auth_get_type (session) != session->key->auth_info_type)
-	    {
-	      gnutls_assert ();
-	      return GNUTLS_E_INVALID_REQUEST;
-	    }
-	}
+        {
+          /* If the credentials for the current authentication scheme,
+           * are not the one we want to set, then it's an error.
+           * This may happen if a rehandshake is performed an the
+           * ciphersuite which is negotiated has different authentication
+           * schema.
+           */
+          if (gnutls_auth_get_type (session) != session->key->auth_info_type)
+            {
+              gnutls_assert ();
+              return GNUTLS_E_INVALID_REQUEST;
+            }
+        }
       else
-	{
-	  /* The new behaviour: Here we reallocate the auth info structure
-	   * in order to be able to negotiate different authentication
-	   * types. Ie. perform an auth_anon and then authenticate again using a
-	   * certificate (in order to prevent revealing the certificate's contents,
-	   * to passive eavesdropers.
-	   */
-	  if (gnutls_auth_get_type (session) != session->key->auth_info_type)
-	    {
+        {
+          /* The new behaviour: Here we reallocate the auth info structure
+           * in order to be able to negotiate different authentication
+           * types. Ie. perform an auth_anon and then authenticate again using a
+           * certificate (in order to prevent revealing the certificate's contents,
+           * to passive eavesdropers.
+           */
+          if (gnutls_auth_get_type (session) != session->key->auth_info_type)
+            {
 
-	      _gnutls_free_auth_info (session);
+              _gnutls_free_auth_info (session);
 
-	      session->key->auth_info = calloc (1, size);
-	      if (session->key->auth_info == NULL)
-		{
-		  gnutls_assert ();
-		  return GNUTLS_E_MEMORY_ERROR;
-		}
+              session->key->auth_info = calloc (1, size);
+              if (session->key->auth_info == NULL)
+                {
+                  gnutls_assert ();
+                  return GNUTLS_E_MEMORY_ERROR;
+                }
 
-	      session->key->auth_info_type = type;
-	      session->key->auth_info_size = size;
-	    }
-	}
+              session->key->auth_info_type = type;
+              session->key->auth_info_size = size;
+            }
+        }
     }
   return 0;
 }
