@@ -89,12 +89,6 @@ _gnutls_record_buffer_put (gnutls_session_t session,
  * This function checks if there are any data to receive in the gnutls
  * buffers.
  *
- * Note that you could also use select() to check for data in a TCP
- * connection, instead of this function.  GnuTLS leaves some data in
- * the tcp buffer in order for select to work. However the select() 
- * alternative is not recommended and will be deprecated in later
- * GnuTLS revisions.
- *
  * Returns: the size of that data or 0.
  **/
 size_t
@@ -739,7 +733,6 @@ parse_handshake_header (gnutls_session_t session, mbuffer_st* bufel, gnutls_hand
     return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
   dataptr = _mbuffer_get_udata_ptr(bufel);
-  data_size = _mbuffer_get_udata_size(bufel) - handshake_header_size;
 
   /* if reading a client hello of SSLv2 */
   if (!IS_DTLS(session) && htype == GNUTLS_HANDSHAKE_CLIENT_HELLO &&
@@ -779,9 +772,10 @@ parse_handshake_header (gnutls_session_t session, mbuffer_st* bufel, gnutls_hand
         {
           hsk->sequence = 0;
           hsk->start_offset = 0;
-          hsk->end_offset = _mbuffer_get_udata_size(bufel) - handshake_header_size;
+          hsk->end_offset = hsk->length;
         }
     }
+  data_size = _mbuffer_get_udata_size(bufel) - handshake_header_size;
 
   /* make the length offset */
   if (hsk->end_offset > 0) hsk->end_offset--;
@@ -1024,10 +1018,9 @@ parse_record_buffered_msgs (gnutls_session_t session,
               ret = _gnutls_buffer_append_data(&recv_buf[0].data, _mbuffer_get_udata_ptr(bufel), data_size);
               if (ret < 0)
                 return gnutls_assert_val(ret);
-
+              _mbuffer_set_uhead_size(bufel, 0);
               _mbuffer_head_remove_bytes(&session->internals.record_buffer, data_size+header_size);
 
-
               if (cmp_hsk_types(htype, recv_buf[0].htype) == 0)
                 { /* an unexpected packet */
                   hsk->htype = recv_buf[0].htype;
@@ -1147,8 +1140,16 @@ _gnutls_handshake_io_recv_int (gnutls_session_t session,
    * trying to receive.
    */
   ret = parse_record_buffered_msgs(session, htype, hsk);
-  if (ret >= 0)
-    return ret;
+  if (IS_DTLS(session))
+    {
+      if (ret >= 0)
+        return ret;
+    }
+  else
+    {
+      if ((ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE && ret < 0) || ret >= 0)
+        return gnutls_assert_val(ret);
+    }
 
   /* if we don't have a complete message waiting for us, try 
    * receiving more */
