@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000, 2001, 2004, 2005, 2007, 2008, 2010 Free Software
+ * Copyright (C) 2000, 2001, 2004, 2005, 2007, 2008, 2010, 2011 Free Software
  * Foundation, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
@@ -37,6 +37,7 @@ digest_length (gnutls_digest_algorithm_t algo)
   switch (algo)
     {
     case GNUTLS_DIG_NULL:
+    case GNUTLS_MAC_AEAD:
       return 0;
     case GNUTLS_DIG_MD5:
     case GNUTLS_DIG_MD2:
@@ -79,6 +80,7 @@ _gnutls_hash_init (digest_hd_st * dig, gnutls_digest_algorithm_t algorithm)
 
       dig->hash = cc->hash;
       dig->copy = cc->copy;
+      dig->reset = cc->reset;
       dig->output = cc->output;
       dig->deinit = cc->deinit;
 
@@ -94,12 +96,28 @@ _gnutls_hash_init (digest_hd_st * dig, gnutls_digest_algorithm_t algorithm)
 
   dig->hash = _gnutls_digest_ops.hash;
   dig->copy = _gnutls_digest_ops.copy;
+  dig->reset = _gnutls_digest_ops.reset;
   dig->output = _gnutls_digest_ops.output;
   dig->deinit = _gnutls_digest_ops.deinit;
 
   return 0;
 }
 
+void
+_gnutls_hash_deinit (digest_hd_st * handle, void *digest)
+{
+  if (handle->handle == NULL)
+    {
+      return;
+    }
+
+  if (digest != NULL)
+    _gnutls_hash_output (handle, digest);
+
+  handle->deinit (handle->handle);
+  handle->handle = NULL;
+}
+
 /* returns the output size of the given hash/mac algorithm
  */
 int
@@ -108,16 +126,6 @@ _gnutls_hash_get_algo_len (gnutls_digest_algorithm_t algorithm)
   return digest_length (algorithm);
 }
 
-int
-_gnutls_hash (digest_hd_st * handle, const void *text, size_t textlen)
-{
-  if (textlen > 0)
-    {
-      handle->hash (handle->handle, text, textlen);
-    }
-  return 0;
-}
-
 int
 _gnutls_hash_copy (digest_hd_st * dst, digest_hd_st * src)
 {
@@ -133,35 +141,7 @@ _gnutls_hash_copy (digest_hd_st * dst, digest_hd_st * src)
   return src->copy (&dst->handle, src->handle);
 }
 
-/* when the current output is needed without calling deinit
- */
-void
-_gnutls_hash_output (digest_hd_st * handle, void *digest)
-{
-  size_t maclen;
 
-  maclen = _gnutls_hash_get_algo_len (handle->algorithm);
-
-  if (digest != NULL)
-    {
-      handle->output (handle->handle, digest, maclen);
-    }
-}
-
-void
-_gnutls_hash_deinit (digest_hd_st * handle, void *digest)
-{
-  if (handle->handle == NULL)
-    {
-      return;
-    }
-
-  if (digest != NULL)
-    _gnutls_hash_output (handle, digest);
-
-  handle->deinit (handle->handle);
-  handle->handle = NULL;
-}
 
 int
 _gnutls_hash_fast (gnutls_digest_algorithm_t algorithm,
@@ -192,12 +172,6 @@ _gnutls_hash_fast (gnutls_digest_algorithm_t algorithm,
 
 /* HMAC interface */
 
-int
-_gnutls_hmac_get_algo_len (gnutls_mac_algorithm_t algorithm)
-{
-  return digest_length (algorithm);
-}
-
 int
 _gnutls_hmac_fast (gnutls_mac_algorithm_t algorithm, const void *key,
                    int keylen, const void *text, size_t textlen, void *digest)
@@ -256,6 +230,7 @@ _gnutls_hmac_init (digest_hd_st * dig, gnutls_mac_algorithm_t algorithm,
       dig->hash = cc->hash;
       dig->output = cc->output;
       dig->deinit = cc->deinit;
+      dig->reset = cc->reset;
 
       return 0;
     }
@@ -270,6 +245,7 @@ _gnutls_hmac_init (digest_hd_st * dig, gnutls_mac_algorithm_t algorithm,
   dig->hash = _gnutls_mac_ops.hash;
   dig->output = _gnutls_mac_ops.output;
   dig->deinit = _gnutls_mac_ops.deinit;
+  dig->reset = _gnutls_mac_ops.reset;
 
   if (_gnutls_mac_ops.setkey (dig->handle, key, keylen) < 0)
     {
@@ -281,29 +257,6 @@ _gnutls_hmac_init (digest_hd_st * dig, gnutls_mac_algorithm_t algorithm,
   return 0;
 }
 
-int
-_gnutls_hmac (digest_hd_st * handle, const void *text, size_t textlen)
-{
-  if (textlen > 0)
-    {
-      return handle->hash (handle->handle, text, textlen);
-    }
-  return 0;
-}
-
-void
-_gnutls_hmac_output (digest_hd_st * handle, void *digest)
-{
-  int maclen;
-
-  maclen = _gnutls_hmac_get_algo_len (handle->algorithm);
-
-  if (digest != NULL)
-    {
-      handle->output (handle->handle, digest, maclen);
-    }
-}
-
 void
 _gnutls_hmac_deinit (digest_hd_st * handle, void *digest)
 {
@@ -333,7 +286,6 @@ get_padsize (gnutls_mac_algorithm_t algorithm)
     }
 }
 
-
 /* Special functions for SSL3 MAC
  */
 
@@ -371,7 +323,26 @@ _gnutls_mac_init_ssl3 (digest_hd_st * ret, gnutls_mac_algorithm_t algorithm,
 }
 
 void
-_gnutls_mac_deinit_ssl3 (digest_hd_st * handle, void *digest)
+_gnutls_mac_reset_ssl3 (digest_hd_st * handle)
+{
+  opaque ipad[48];
+  int padsize;
+
+  padsize = get_padsize (handle->algorithm);
+
+  memset (ipad, 0x36, padsize);
+
+  _gnutls_hash_reset(handle);
+
+  if (handle->keysize > 0)
+    _gnutls_hash (handle, handle->key, handle->keysize);
+  _gnutls_hash (handle, ipad, padsize);
+
+  return;
+}
+
+int 
+_gnutls_mac_output_ssl3 (digest_hd_st * handle, void *digest)
 {
   opaque ret[MAX_HASH_SIZE];
   digest_hd_st td;
@@ -383,8 +354,7 @@ _gnutls_mac_deinit_ssl3 (digest_hd_st * handle, void *digest)
   if (padsize == 0)
     {
       gnutls_assert ();
-      _gnutls_hash_deinit (handle, NULL);
-      return;
+      return GNUTLS_E_INTERNAL_ERROR;
     }
 
   memset (opad, 0x5C, padsize);
@@ -393,8 +363,7 @@ _gnutls_mac_deinit_ssl3 (digest_hd_st * handle, void *digest)
   if (rc < 0)
     {
       gnutls_assert ();
-      _gnutls_hash_deinit (handle, NULL);
-      return;
+      return rc;
     }
 
   if (handle->keysize > 0)
@@ -402,15 +371,26 @@ _gnutls_mac_deinit_ssl3 (digest_hd_st * handle, void *digest)
 
   _gnutls_hash (&td, opad, padsize);
   block = _gnutls_hmac_get_algo_len (handle->algorithm);
-  _gnutls_hash_deinit (handle, ret);    /* get the previous hash */
+  _gnutls_hash_output (handle, ret);    /* get the previous hash */
   _gnutls_hash (&td, ret, block);
 
   _gnutls_hash_deinit (&td, digest);
-
-  return;
+  
+  return 0;
 }
 
-void
+int
+_gnutls_mac_deinit_ssl3 (digest_hd_st * handle, void *digest)
+{
+int ret = 0;
+
+  if (digest != NULL) ret = _gnutls_mac_output_ssl3(handle, digest);
+  _gnutls_hash_deinit(handle, NULL);
+  
+  return ret;
+}
+
+int
 _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
                                    void *digest, opaque * key,
                                    uint32_t key_size)
@@ -426,7 +406,8 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
   if (padsize == 0)
     {
       gnutls_assert ();
-      return;
+      rc = GNUTLS_E_INTERNAL_ERROR;
+      goto cleanup;
     }
 
   memset (opad, 0x5C, padsize);
@@ -436,7 +417,7 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
   if (rc < 0)
     {
       gnutls_assert ();
-      return;
+      goto cleanup;
     }
 
   if (key_size > 0)
@@ -454,7 +435,11 @@ _gnutls_mac_deinit_ssl3_handshake (digest_hd_st * handle,
 
   _gnutls_hash_deinit (&td, digest);
 
-  return;
+  return 0;
+
+cleanup:
+  _gnutls_hash_deinit(handle, NULL);
+  return rc;
 }
 
 static int
@@ -486,6 +471,9 @@ ssl3_sha (int i, opaque * secret, int secret_len,
   return 0;
 }
 
+#define SHA1_DIGEST_OUTPUT 20
+#define MD5_DIGEST_OUTPUT 16
+
 static int
 ssl3_md5 (int i, opaque * secret, int secret_len,
           opaque * rnd, int rnd_len, void *digest)
@@ -511,7 +499,7 @@ ssl3_md5 (int i, opaque * secret, int secret_len,
       return ret;
     }
 
-  _gnutls_hash (&td, tmp, _gnutls_hash_get_algo_len (GNUTLS_MAC_SHA1));
+  _gnutls_hash (&td, tmp, SHA1_DIGEST_OUTPUT);
 
   _gnutls_hash_deinit (&td, digest);
   return 0;
@@ -524,7 +512,7 @@ _gnutls_ssl3_hash_md5 (const void *first, int first_len,
 {
   opaque digest[MAX_HASH_SIZE];
   digest_hd_st td;
-  int block = _gnutls_hash_get_algo_len (GNUTLS_MAC_MD5);
+  int block = MD5_DIGEST_OUTPUT;
   int rc;
 
   rc = _gnutls_hash_init (&td, GNUTLS_MAC_MD5);
@@ -558,7 +546,7 @@ _gnutls_ssl3_generate_random (void *secret, int secret_len,
 {
   int i = 0, copy, output_bytes;
   opaque digest[MAX_HASH_SIZE];
-  int block = _gnutls_hash_get_algo_len (GNUTLS_MAC_MD5);
+  int block = MD5_DIGEST_OUTPUT;
   int result, times;
 
   output_bytes = 0;
