@@ -39,7 +39,6 @@
 #include <gnutls_datum.h>
 #include <gnutls_rsa_export.h>
 #include <gnutls_mbuffers.h>
-#include "../libextra/ext_inner_application.h"  /* isn't this too much? */
 
 /* This is a temporary function to be used before the generate_*
    internal API is changed to use mbuffers. For now we don't avoid the
@@ -59,7 +58,7 @@ send_handshake (gnutls_session_t session, opaque * data, size_t size,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  bufel = _gnutls_handshake_alloc (size, size);
+  bufel = _gnutls_handshake_alloc(session, size, size);
   if (bufel == NULL)
     {
       gnutls_assert ();
@@ -76,6 +75,7 @@ send_handshake (gnutls_session_t session, opaque * data, size_t size,
  */
 
 #define MASTER_SECRET "master secret"
+#define MASTER_SECRET_SIZE (sizeof(MASTER_SECRET)-1)
 static int generate_normal_master (gnutls_session_t session, int);
 
 int
@@ -135,14 +135,11 @@ generate_normal_master (gnutls_session_t session, int keep_premaster)
 
       ret =
         _gnutls_PRF (session, PREMASTER.data, PREMASTER.size,
-                     MASTER_SECRET, strlen (MASTER_SECRET),
+                     MASTER_SECRET, MASTER_SECRET_SIZE,
                      rnd, 2 * GNUTLS_RANDOM_SIZE, GNUTLS_MASTER_SIZE,
                      session->security_parameters.master_secret);
     }
 
-  /* TLS/IA inner secret is derived from the master secret. */
-  _gnutls_ia_derive_inner_secret (session);
-
   if (!keep_premaster)
     _gnutls_free_datum (&PREMASTER);
 
@@ -166,45 +163,44 @@ generate_normal_master (gnutls_session_t session, int keep_premaster)
 int
 _gnutls_send_server_kx_message (gnutls_session_t session, int again)
 {
-  uint8_t *data = NULL;
-  int data_size = 0;
+  gnutls_buffer_st data;
   int ret = 0;
 
   if (session->internals.auth_struct->gnutls_generate_server_kx == NULL)
     return 0;
 
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
-      data_size =
+      ret =
         session->internals.auth_struct->gnutls_generate_server_kx (session,
                                                                    &data);
 
-      if (data_size == GNUTLS_E_INT_RET_0)
+      if (ret == GNUTLS_E_INT_RET_0)
         {
           gnutls_assert ();
-          return 0;
+          ret = 0;
+          goto cleanup;
         }
 
-      if (data_size < 0)
+      if (ret < 0)
         {
           gnutls_assert ();
-          return data_size;
+          goto cleanup;
         }
     }
 
-  ret = send_handshake (session, data, data_size,
+  ret = send_handshake (session, data.data, data.length,
                         GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE);
-  gnutls_free (data);
-
   if (ret < 0)
     {
       gnutls_assert ();
-      return ret;
     }
-  return data_size;
+  
+cleanup:
+  _gnutls_buffer_clear (&data);
+  return ret;
 }
 
 /* This function sends a certificate request message to the
@@ -213,8 +209,7 @@ _gnutls_send_server_kx_message (gnutls_session_t session, int again)
 int
 _gnutls_send_server_certificate_request (gnutls_session_t session, int again)
 {
-  uint8_t *data = NULL;
-  int data_size = 0;
+  gnutls_buffer_st data;
   int ret = 0;
 
   if (session->internals.
@@ -224,32 +219,32 @@ _gnutls_send_server_certificate_request (gnutls_session_t session, int again)
   if (session->internals.send_cert_req <= 0)
     return 0;
 
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
-      data_size =
+      ret =
         session->internals.
         auth_struct->gnutls_generate_server_certificate_request (session,
                                                                  &data);
 
-      if (data_size < 0)
+      if (ret < 0)
         {
           gnutls_assert ();
-          return data_size;
+          goto cleanup;
         }
     }
-  ret = send_handshake (session, data, data_size,
-                        GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST);
-  gnutls_free (data);
 
+  ret = send_handshake (session, data.data, data.length,
+                        GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST);
   if (ret < 0)
     {
       gnutls_assert ();
-      return ret;
     }
-  return data_size;
+  
+cleanup:
+  _gnutls_buffer_clear (&data);
+  return ret;
 }
 
 
@@ -259,38 +254,34 @@ _gnutls_send_server_certificate_request (gnutls_session_t session, int again)
 int
 _gnutls_send_client_kx_message (gnutls_session_t session, int again)
 {
-  uint8_t *data;
-  int data_size;
+  gnutls_buffer_st data;
   int ret = 0;
 
   if (session->internals.auth_struct->gnutls_generate_client_kx == NULL)
     return 0;
 
-
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
-      data_size =
+      ret =
         session->internals.auth_struct->gnutls_generate_client_kx (session,
                                                                    &data);
-      if (data_size < 0)
+      if (ret < 0)
         {
-          gnutls_assert ();
-          return data_size;
+          gnutls_assert();
+          goto cleanup;
         }
     }
-  ret = send_handshake (session, data, data_size,
+  ret = send_handshake (session, data.data, data.length,
                         GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE);
-  gnutls_free (data);
-
   if (ret < 0)
     {
       gnutls_assert ();
-      return ret;
     }
-
+  
+cleanup:
+  _gnutls_buffer_clear (&data);
   return ret;
 }
 
@@ -301,9 +292,8 @@ _gnutls_send_client_kx_message (gnutls_session_t session, int again)
 int
 _gnutls_send_client_certificate_verify (gnutls_session_t session, int again)
 {
-  uint8_t *data;
+  gnutls_buffer_st data;
   int ret = 0;
-  int data_size;
 
   /* This is a packet that is only sent by the client
    */
@@ -315,6 +305,7 @@ _gnutls_send_client_certificate_verify (gnutls_session_t session, int again)
   if (session->key->certificate_requested == 0)
     return 0;
 
+
   if (session->internals.auth_struct->gnutls_generate_client_cert_vrfy ==
       NULL)
     {
@@ -323,157 +314,42 @@ _gnutls_send_client_certificate_verify (gnutls_session_t session, int again)
                                  */
     }
 
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
-      data_size =
+      ret =
         session->internals.
         auth_struct->gnutls_generate_client_cert_vrfy (session, &data);
-      if (data_size < 0)
+      if (ret < 0)
         {
-          gnutls_assert ();
-          return data_size;
+          gnutls_assert();
+          goto cleanup;
         }
-      if (data_size == 0)
-        return 0;
+
+      if (ret == 0)
+          goto cleanup;
 
     }
-  ret = send_handshake (session, data, data_size,
+  ret = send_handshake (session, data.data, data.length,
                         GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY);
-  gnutls_free (data);
 
-  return ret;
-}
-
-
-int
-_gnutls_recv_server_kx_message (gnutls_session_t session)
-{
-  uint8_t *data = NULL;
-  int datasize;
-  int ret = 0;
-  Optional optflag = MANDATORY_PACKET;
-
-  if (session->internals.auth_struct->gnutls_process_server_kx != NULL)
+  if (ret < 0)
     {
-
-      /* EXCEPTION FOR RSA_EXPORT cipher suite 
-       */
-      if (_gnutls_session_is_export (session) != 0 &&
-          _gnutls_peers_cert_less_512 (session) != 0)
-        {
-          gnutls_assert ();
-          return 0;
-        }
-
-      /* Server key exchange packet is optional for PSK. */
-      if (_gnutls_session_is_psk (session))
-        optflag = OPTIONAL_PACKET;
-
-      ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
-                                GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE,
-                                optflag);
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          return ret;
-        }
-
-      ret =
-        session->internals.auth_struct->gnutls_process_server_kx (session,
-                                                                  data,
-                                                                  datasize);
-      gnutls_free (data);
-
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          return ret;
-        }
-
+      gnutls_assert ();
     }
+  
+cleanup:
+  _gnutls_buffer_clear (&data);
   return ret;
 }
 
-int
-_gnutls_recv_server_certificate_request (gnutls_session_t session)
-{
-  uint8_t *data;
-  int datasize;
-  int ret = 0;
-
-  if (session->internals.
-      auth_struct->gnutls_process_server_certificate_request != NULL)
-    {
-
-      ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
-                                GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST,
-                                OPTIONAL_PACKET);
-      if (ret < 0)
-        return ret;
-
-      if (ret == 0 && datasize == 0)
-        return 0;               /* ignored */
-
-      ret =
-        session->internals.
-        auth_struct->gnutls_process_server_certificate_request (session, data,
-                                                                datasize);
-      gnutls_free (data);
-      if (ret < 0)
-        return ret;
-
-    }
-  return ret;
-}
-
-int
-_gnutls_recv_client_kx_message (gnutls_session_t session)
-{
-  uint8_t *data;
-  int datasize;
-  int ret = 0;
-
-
-  /* Do key exchange only if the algorithm permits it */
-  if (session->internals.auth_struct->gnutls_process_client_kx != NULL)
-    {
-
-      ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
-                                GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE,
-                                MANDATORY_PACKET);
-      if (ret < 0)
-        return ret;
-
-      ret =
-        session->internals.auth_struct->gnutls_process_client_kx (session,
-                                                                  data,
-                                                                  datasize);
-      gnutls_free (data);
-      if (ret < 0)
-        return ret;
-
-    }
-
-  return ret;
-}
-
-
 /* This is called when we want send our certificate
  */
 int
 _gnutls_send_client_certificate (gnutls_session_t session, int again)
 {
-  uint8_t *data = NULL;
-  int data_size = 0;
+  gnutls_buffer_st data;
   int ret = 0;
 
 
@@ -484,8 +360,7 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
       NULL)
     return 0;
 
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
@@ -494,14 +369,14 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
         {
           /* TLS 1.0 or SSL 3.0 with a valid certificate 
            */
-          data_size =
+          ret =
             session->internals.
             auth_struct->gnutls_generate_client_certificate (session, &data);
 
-          if (data_size < 0)
+          if (ret < 0)
             {
-              gnutls_assert ();
-              return data_size;
+              gnutls_assert();
+              goto cleanup;
             }
         }
     }
@@ -521,18 +396,13 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
   else
     {                           /* TLS 1.0 or SSL 3.0 with a valid certificate 
                                  */
-      ret = send_handshake (session, data, data_size,
+      ret = send_handshake (session, data.data, data.length,
                             GNUTLS_HANDSHAKE_CERTIFICATE_PKT);
-      gnutls_free (data);
     }
 
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-
-  return data_size;
+cleanup:
+  _gnutls_buffer_clear (&data);
+  return ret;
 }
 
 
@@ -541,8 +411,7 @@ _gnutls_send_client_certificate (gnutls_session_t session, int again)
 int
 _gnutls_send_server_certificate (gnutls_session_t session, int again)
 {
-  uint8_t *data = NULL;
-  int data_size = 0;
+  gnutls_buffer_st data;
   int ret = 0;
 
 
@@ -550,134 +419,245 @@ _gnutls_send_server_certificate (gnutls_session_t session, int again)
       NULL)
     return 0;
 
-  data = NULL;
-  data_size = 0;
+  _gnutls_buffer_init( &data);
 
   if (again == 0)
     {
-      data_size =
+      ret =
         session->internals.
         auth_struct->gnutls_generate_server_certificate (session, &data);
 
-      if (data_size < 0)
-        {
-          gnutls_assert ();
-          return data_size;
-        }
+        if (ret < 0)
+          {
+            gnutls_assert();
+            goto cleanup;
+          }
     }
-  ret = send_handshake (session, data, data_size,
+  ret = send_handshake (session, data.data, data.length,
                         GNUTLS_HANDSHAKE_CERTIFICATE_PKT);
-  gnutls_free (data);
-
   if (ret < 0)
     {
       gnutls_assert ();
-      return ret;
     }
-
-  return data_size;
+  
+cleanup:
+  _gnutls_buffer_clear (&data);
+  return ret;
 }
 
 
 int
-_gnutls_recv_client_certificate (gnutls_session_t session)
+_gnutls_recv_server_kx_message (gnutls_session_t session)
 {
-  int datasize;
-  opaque *data;
+  gnutls_buffer_st buf;
   int ret = 0;
-  int optional;
+  optional_t optflag = MANDATORY_PACKET;
 
-  if (session->internals.auth_struct->gnutls_process_client_certificate !=
-      NULL)
+  if (session->internals.auth_struct->gnutls_process_server_kx != NULL)
     {
 
-      /* if we have not requested a certificate then just return
+      /* EXCEPTION FOR RSA_EXPORT cipher suite 
        */
-      if (session->internals.send_cert_req == 0)
+      if (_gnutls_session_is_export (session) != 0 &&
+          _gnutls_peers_cert_less_512 (session) != 0)
         {
+          gnutls_assert ();
           return 0;
         }
 
-      if (session->internals.send_cert_req == GNUTLS_CERT_REQUIRE)
-        optional = MANDATORY_PACKET;
-      else
-        optional = OPTIONAL_PACKET;
+      /* Server key exchange packet is optional for PSK. */
+      if (_gnutls_session_is_psk (session))
+        optflag = OPTIONAL_PACKET;
 
       ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
-                                GNUTLS_HANDSHAKE_CERTIFICATE_PKT, optional);
+        _gnutls_recv_handshake (session, 
+                                GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE,
+                                optflag, &buf);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          return ret;
+        }
+
+      ret =
+        session->internals.auth_struct->gnutls_process_server_kx (session,
+                                                                  buf.data,
+                                                                  buf.length);
+      _gnutls_buffer_clear(&buf);
 
       if (ret < 0)
         {
-          /* Handle the case of old SSL3 clients who send
-           * a warning alert instead of an empty certificate to indicate
-           * no certificate.
-           */
-          if (optional == OPTIONAL_PACKET &&
-              ret == GNUTLS_E_WARNING_ALERT_RECEIVED &&
-              gnutls_protocol_get_version (session) == GNUTLS_SSL3 &&
-              gnutls_alert_get (session) == GNUTLS_A_SSL3_NO_CERTIFICATE)
-            {
-
-              /* SSL3 does not send an empty certificate,
-               * but this alert. So we just ignore it.
-               */
-              gnutls_assert ();
-              return 0;
-            }
-
-          /* certificate was required 
-           */
-          if ((ret == GNUTLS_E_WARNING_ALERT_RECEIVED
-               || ret == GNUTLS_E_FATAL_ALERT_RECEIVED)
-              && optional == MANDATORY_PACKET)
-            {
-              gnutls_assert ();
-              return GNUTLS_E_NO_CERTIFICATE_FOUND;
-            }
-
+          gnutls_assert ();
           return ret;
         }
 
-      if (ret == 0 && datasize == 0 && optional == OPTIONAL_PACKET)
+    }
+  return ret;
+}
+
+int
+_gnutls_recv_server_certificate_request (gnutls_session_t session)
+{
+  gnutls_buffer_st buf;
+  int ret = 0;
+
+  if (session->internals.
+      auth_struct->gnutls_process_server_certificate_request != NULL)
+    {
+
+      ret =
+        _gnutls_recv_handshake (session, 
+                                GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST,
+                                OPTIONAL_PACKET, &buf);
+      if (ret < 0)
+        return ret;
+
+      if (ret == 0 && buf.length == 0)
         {
-          /* Client has not sent the certificate message.
-           * well I'm not sure we should accept this
-           * behaviour.
-           */
-          gnutls_assert ();
-          return 0;
+          _gnutls_buffer_clear(&buf);
+          return 0;               /* ignored */
         }
+
       ret =
         session->internals.
-        auth_struct->gnutls_process_client_certificate (session, data,
-                                                        datasize);
+        auth_struct->gnutls_process_server_certificate_request (session, buf.data,
+                                                                buf.length);
+      _gnutls_buffer_clear (&buf);
+      if (ret < 0)
+        return ret;
 
-      gnutls_free (data);
-      if (ret < 0 && ret != GNUTLS_E_NO_CERTIFICATE_FOUND)
-        {
-          gnutls_assert ();
-          return ret;
-        }
+    }
+  return ret;
+}
 
-      /* ok we should expect a certificate verify message now 
-       */
-      if (ret == GNUTLS_E_NO_CERTIFICATE_FOUND && optional == OPTIONAL_PACKET)
-        ret = 0;
-      else
-        session->key->certificate_requested = 1;
+int
+_gnutls_recv_client_kx_message (gnutls_session_t session)
+{
+  gnutls_buffer_st buf;
+  int ret = 0;
+
+
+  /* Do key exchange only if the algorithm permits it */
+  if (session->internals.auth_struct->gnutls_process_client_kx != NULL)
+    {
+
+      ret =
+        _gnutls_recv_handshake (session, 
+                                GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE,
+                                MANDATORY_PACKET, &buf);
+      if (ret < 0)
+        return ret;
+
+      ret =
+        session->internals.auth_struct->gnutls_process_client_kx (session,
+                                                                  buf.data,
+                                                                  buf.length);
+      _gnutls_buffer_clear (&buf);
+      if (ret < 0)
+        return ret;
 
     }
 
   return ret;
 }
 
+
+int
+_gnutls_recv_client_certificate (gnutls_session_t session)
+{
+  gnutls_buffer_st buf;
+  int ret = 0;
+  int optional;
+
+  if (session->internals.auth_struct->gnutls_process_client_certificate ==
+      NULL)
+    return 0;
+
+  /* if we have not requested a certificate then just return
+   */
+  if (session->internals.send_cert_req == 0)
+    {
+      return 0;
+    }
+
+  if (session->internals.send_cert_req == GNUTLS_CERT_REQUIRE)
+    optional = MANDATORY_PACKET;
+  else
+    optional = OPTIONAL_PACKET;
+
+  ret =
+    _gnutls_recv_handshake (session, GNUTLS_HANDSHAKE_CERTIFICATE_PKT, 
+      optional, &buf);
+
+  if (ret < 0)
+    {
+      /* Handle the case of old SSL3 clients who send
+       * a warning alert instead of an empty certificate to indicate
+       * no certificate.
+       */
+      if (optional == OPTIONAL_PACKET &&
+          ret == GNUTLS_E_WARNING_ALERT_RECEIVED &&
+          gnutls_protocol_get_version (session) == GNUTLS_SSL3 &&
+          gnutls_alert_get (session) == GNUTLS_A_SSL3_NO_CERTIFICATE)
+        {
+
+          /* SSL3 does not send an empty certificate,
+           * but this alert. So we just ignore it.
+           */
+          gnutls_assert ();
+          return 0;
+        }
+
+      /* certificate was required 
+       */
+      if ((ret == GNUTLS_E_WARNING_ALERT_RECEIVED
+           || ret == GNUTLS_E_FATAL_ALERT_RECEIVED)
+          && optional == MANDATORY_PACKET)
+        {
+          gnutls_assert ();
+          return GNUTLS_E_NO_CERTIFICATE_FOUND;
+        }
+
+      return ret;
+    }
+
+  if (ret == 0 && buf.length == 0 && optional == OPTIONAL_PACKET)
+    {
+      /* Client has not sent the certificate message.
+       * well I'm not sure we should accept this
+       * behaviour.
+       */
+      gnutls_assert ();
+      ret = 0;
+      goto cleanup;
+    }
+  ret =
+    session->internals.
+    auth_struct->gnutls_process_client_certificate (session, buf.data,
+                                                    buf.length);
+
+  if (ret < 0 && ret != GNUTLS_E_NO_CERTIFICATE_FOUND)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  /* ok we should expect a certificate verify message now 
+   */
+  if (ret == GNUTLS_E_NO_CERTIFICATE_FOUND && optional == OPTIONAL_PACKET)
+    ret = 0;
+  else
+    session->key->certificate_requested = 1;
+
+cleanup:
+  _gnutls_buffer_clear(&buf);
+  return ret;
+}
+
 int
 _gnutls_recv_server_certificate (gnutls_session_t session)
 {
-  int datasize;
-  opaque *data;
+  gnutls_buffer_st buf;
   int ret = 0;
 
   if (session->internals.auth_struct->gnutls_process_server_certificate !=
@@ -685,10 +665,9 @@ _gnutls_recv_server_certificate (gnutls_session_t session)
     {
 
       ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
+        _gnutls_recv_handshake (session, 
                                 GNUTLS_HANDSHAKE_CERTIFICATE_PKT,
-                                MANDATORY_PACKET);
+                                MANDATORY_PACKET, &buf);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -697,9 +676,9 @@ _gnutls_recv_server_certificate (gnutls_session_t session)
 
       ret =
         session->internals.
-        auth_struct->gnutls_process_server_certificate (session, data,
-                                                        datasize);
-      gnutls_free (data);
+        auth_struct->gnutls_process_server_certificate (session, buf.data,
+                                                        buf.length);
+      _gnutls_buffer_clear(&buf);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -717,45 +696,41 @@ _gnutls_recv_server_certificate (gnutls_session_t session)
 int
 _gnutls_recv_client_certificate_verify_message (gnutls_session_t session)
 {
-  uint8_t *data;
-  int datasize;
+  gnutls_buffer_st buf;
   int ret = 0;
 
 
-  if (session->internals.auth_struct->gnutls_process_client_cert_vrfy != NULL)
+  if (session->internals.auth_struct->gnutls_process_client_cert_vrfy == NULL)
+    return 0;
+
+  if (session->internals.send_cert_req == 0 ||
+      session->key->certificate_requested == 0)
     {
-
-      if (session->internals.send_cert_req == 0 ||
-          session->key->certificate_requested == 0)
-        {
-          return 0;
-        }
-
-      ret =
-        _gnutls_recv_handshake (session, &data,
-                                &datasize,
-                                GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY,
-                                OPTIONAL_PACKET);
-      if (ret < 0)
-        return ret;
-
-      if (ret == 0 && datasize == 0
-          && session->internals.send_cert_req == GNUTLS_CERT_REQUIRE)
-        {
-          /* certificate was required */
-          gnutls_assert ();
-          return GNUTLS_E_NO_CERTIFICATE_FOUND;
-        }
-
-      ret =
-        session->internals.
-        auth_struct->gnutls_process_client_cert_vrfy (session, data,
-                                                      datasize);
-      gnutls_free (data);
-      if (ret < 0)
-        return ret;
-
+      return 0;
     }
 
+  ret =
+    _gnutls_recv_handshake (session, 
+                            GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY,
+                            OPTIONAL_PACKET, &buf);
+  if (ret < 0)
+    return ret;
+
+  if (ret == 0 && buf.length == 0
+      && session->internals.send_cert_req == GNUTLS_CERT_REQUIRE)
+    {
+      /* certificate was required */
+      gnutls_assert ();
+      ret = GNUTLS_E_NO_CERTIFICATE_FOUND;
+      goto cleanup;
+    }
+
+  ret =
+    session->internals.
+    auth_struct->gnutls_process_client_cert_vrfy (session, buf.data,
+                                                  buf.length);
+
+cleanup:
+  _gnutls_buffer_clear(&buf);
   return ret;
 }
