@@ -29,13 +29,13 @@
 
 #include <gnutls_int.h>
 #ifdef ENABLE_SRP
-#include <auth_srp.h>
+#include <auth/srp.h>
 #endif
 #ifdef ENABLE_PSK
-#include <auth_psk.h>
+#include <auth/psk.h>
 #endif
-#include <auth_anon.h>
-#include <auth_cert.h>
+#include <auth/anon.h>
+#include <auth/cert.h>
 #include <gnutls_errors.h>
 #include <gnutls_auth.h>
 #include <gnutls_session_pack.h>
@@ -803,7 +803,7 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
 {
   size_t pack_size;
   int ret;
-  time_t timestamp = gnutls_time (0);
+  time_t timestamp = time (0);
 
   BUFFER_POP_NUM (ps, pack_size);
 
@@ -827,20 +827,20 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_POP_NUM (ps, session->internals.resumed_security_parameters.version);
 
   BUFFER_POP (ps,
-              session->internals.resumed_security_parameters.master_secret,
+              &session->internals.resumed_security_parameters.master_secret,
               GNUTLS_MASTER_SIZE);
 
   BUFFER_POP (ps,
-              session->internals.resumed_security_parameters.client_random,
+              &session->internals.resumed_security_parameters.client_random,
               GNUTLS_RANDOM_SIZE);
   BUFFER_POP (ps,
-              session->internals.resumed_security_parameters.server_random,
+              &session->internals.resumed_security_parameters.server_random,
               GNUTLS_RANDOM_SIZE);
   BUFFER_POP_NUM (ps,
                   session->internals.
                   resumed_security_parameters.session_id_size);
 
-  BUFFER_POP (ps, session->internals.resumed_security_parameters.session_id,
+  BUFFER_POP (ps, &session->internals.resumed_security_parameters.session_id,
               session->internals.resumed_security_parameters.session_id_size);
 
   BUFFER_POP_NUM (ps,
