@@ -41,9 +41,9 @@
 #include <gnutls_mbuffers.h>
 #include <gnutls_state.h>
 #include <gnutls_constate.h>
-#include <auth_cert.h>
-#include <auth_anon.h>
-#include <auth_psk.h>
+#include <auth/cert.h>
+#include <auth/anon.h>
+#include <auth/psk.h>
 #include <gnutls_algorithms.h>
 #include <gnutls_rsa_export.h>
 #include <gnutls_extensions.h>
@@ -187,7 +187,7 @@ _gnutls_session_cert_type_supported (gnutls_session_t session,
         {
           for (i = 0; i < cred->ncerts; i++)
             {
-              if (cred->cert_list[i][0].cert_type == cert_type)
+              if (cred->cert_list[i][0].type == cert_type)
                 {
                   cert_found = 1;
                   break;
@@ -764,9 +764,8 @@ _gnutls_record_set_default_version (gnutls_session_t session,
  * suites (the ones that start with 0xFF).  By default or if @allow
  * is 0 then these cipher suites will not be advertized nor used.
  *
- * Unless this function is called with the option to allow (1), then
- * no compression algorithms, like LZO.  That is because these
- * algorithms are not yet defined in any RFC or even internet draft.
+ * Currently GnuTLS does not include such cipher-suites or
+ * compression algorithms.
  *
  * Enabling the private ciphersuites when talking to other than
  * gnutls servers and clients may cause interoperability problems.
