@@ -28,7 +28,6 @@
 #include <gnutls_num.h>
 #include <gnutls_str.h>
 #include <stdarg.h>
-#include <c-ctype.h>
 
 /* These function are like strcat, strcpy. They only
  * do bound checking (they shouldn't cause buffer overruns),
@@ -101,6 +100,13 @@ _gnutls_buffer_init (gnutls_buffer_st * str)
   str->length = 0;
 }
 
+void _gnutls_buffer_replace_data( gnutls_buffer_st * buf, gnutls_datum_t * data)
+{
+    gnutls_free(buf->allocd);
+    buf->allocd = buf->data = data->data;
+    buf->max_length = buf->length = data->size;
+}
+
 void
 _gnutls_buffer_clear (gnutls_buffer_st * str)
 {
@@ -121,6 +127,8 @@ _gnutls_buffer_append_data (gnutls_buffer_st * dest, const void *data,
 {
   size_t tot_len = data_size + dest->length;
 
+  if (data_size == 0) return 0;
+
   if (dest->max_length >= tot_len)
     {
       size_t unused = MEMSUB (dest->data, dest->allocd);
@@ -344,7 +352,7 @@ _gnutls_buffer_delete_data (gnutls_buffer_st * dest, int pos, size_t str_size)
 
 
 int
-_gnutls_buffer_escape (gnutls_buffer_st * dest, int all,
+_gnutls_buffer_escape (gnutls_buffer_st * dest,
                        const char *const invalid_chars)
 {
   int rv = -1;
@@ -354,8 +362,8 @@ _gnutls_buffer_escape (gnutls_buffer_st * dest, int all,
   while (pos < dest->length)
     {
 
-      if (all != 0 || (dest->data[pos] == '\\' || strchr (invalid_chars, dest->data[pos])
-          || !c_isgraph (dest->data[pos])))
+      if (dest->data[pos] == '\\' || strchr (invalid_chars, dest->data[pos])
+          || !isgraph (dest->data[pos]))
         {
 
           snprintf (t, sizeof (t), "%%%.2X", (unsigned int) dest->data[pos]);
@@ -367,10 +375,9 @@ _gnutls_buffer_escape (gnutls_buffer_st * dest, int all,
               rv = -1;
               goto cleanup;
             }
-          pos+=3;
+
         }
-      else
-        pos++;
+      pos++;
     }
 
   rv = 0;
@@ -522,20 +529,14 @@ _gnutls_hex2bin (const opaque * hex_data, int hex_size, opaque * bin_data,
  * return 1 on success or 0 on error
  *
  * note: certnamesize is required as X509 certs can contain embedded NULs in
- * the strings such as CN or subjectAltName.
- *
- * @level: is used for recursion. Use 0 when you call this function.
+ * the strings such as CN or subjectAltName
  */
 int
 _gnutls_hostname_compare (const char *certname,
-                          size_t certnamesize, const char *hostname, int level)
+                          size_t certnamesize, const char *hostname)
 {
-
-  if (level > 5)
-    return 0;
-
   /* find the first different character */
-  for (; *certname && *hostname && c_toupper (*certname) == c_toupper (*hostname);
+  for (; *certname && *hostname && toupper (*certname) == toupper (*hostname);
        certname++, hostname++, certnamesize--)
     ;
 
@@ -553,7 +554,7 @@ _gnutls_hostname_compare (const char *certname,
       while (1)
         {
           /* Use a recursive call to allow multiple wildcards */
-          if (_gnutls_hostname_compare (certname, certnamesize, hostname, level+1))
+          if (_gnutls_hostname_compare (certname, certnamesize, hostname))
             return 1;
 
           /* wildcards are only allowed to match a single domain
@@ -570,11 +571,34 @@ _gnutls_hostname_compare (const char *certname,
 }
 
 int
-_gnutls_buffer_append_prefix (gnutls_buffer_st * buf, size_t data_size)
+_gnutls_buffer_append_prefix (gnutls_buffer_st * buf, int pfx_size, size_t data_size)
 {
   opaque ss[4];
-  _gnutls_write_uint32 (data_size, ss);
-  return _gnutls_buffer_append_data (buf, ss, 4);
+
+  if (pfx_size == 32)
+    {
+      _gnutls_write_uint32 (data_size, ss);
+      pfx_size = 4;
+    }
+  else if (pfx_size == 24)
+    {
+      _gnutls_write_uint24 (data_size, ss);
+      pfx_size = 3;
+    }
+  else if (pfx_size == 16)
+    {
+      _gnutls_write_uint16 (data_size, ss);
+      pfx_size = 2;
+    }
+  else if (pfx_size == 8)
+    {
+      ss[0] = data_size;
+      pfx_size = 1;
+    }
+  else
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  return _gnutls_buffer_append_data (buf, ss, pfx_size);
 }
 
 /* Reads an uint32 number from the buffer. If check is non zero it will also check whether
@@ -641,14 +665,44 @@ _gnutls_buffer_pop_datum_prefix (gnutls_buffer_st * buf,
 }
 
 int
-_gnutls_buffer_append_data_prefix (gnutls_buffer_st * buf, const void *data,
-                                   size_t data_size)
+_gnutls_buffer_append_data_prefix (gnutls_buffer_st * buf, 
+    int pfx_size, const void *data, size_t data_size)
 {
-  _gnutls_buffer_append_prefix (buf, data_size);
-  if (data_size > 0)
-    return _gnutls_buffer_append_data (buf, data, data_size);
+int ret = 0, ret1;
 
-  return 0;
+  ret1 = _gnutls_buffer_append_prefix (buf, pfx_size, data_size);
+  if (ret1 < 0)
+    return gnutls_assert_val(ret1);
+
+  if (data_size > 0)
+    {
+      ret = _gnutls_buffer_append_data (buf, data, data_size);
+
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+    }
+
+  return ret + ret1;
+}
+
+int _gnutls_buffer_append_mpi (gnutls_buffer_st * buf, int pfx_size, bigint_t mpi, int lz)
+{
+gnutls_datum_t dd;
+int ret;
+
+  if (lz)
+    ret = _gnutls_mpi_dprint_lz (mpi, &dd);
+  else
+    ret = _gnutls_mpi_dprint (mpi, &dd);
+
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  ret = _gnutls_buffer_append_data_prefix(buf, pfx_size, dd.data, dd.size);
+  
+  _gnutls_free_datum(&dd);
+  
+  return ret;
 }
 
 int
