@@ -28,10 +28,10 @@
  */
 
 #include <gnutls_int.h>
-#include <auth_srp.h>
-#include <auth_anon.h>
-#include <auth_cert.h>
-#include <auth_psk.h>
+#include <auth/srp.h>
+#include <auth/anon.h>
+#include <auth/cert.h>
+#include <auth/psk.h>
 #include <gnutls_errors.h>
 #include <gnutls_auth.h>
 #include <gnutls_state.h>
@@ -479,7 +479,7 @@ gnutls_certificate_get_ours (gnutls_session_t session)
   if (session->internals.selected_cert_list == NULL)
     return NULL;
 
-  return &session->internals.selected_cert_list[0].raw;
+  return &session->internals.selected_cert_list[0].cert;
 }
 
 /**
