@@ -35,6 +35,9 @@
 #include <nettle/des.h>
 #include <nettle/nettle-meta.h>
 #include <nettle/cbc.h>
+# ifdef NETTLE_GCM
+#include <nettle/gcm.h>
+# endif
 
 /* Functions that refer to the libgcrypt library.
  */
@@ -42,15 +45,19 @@
 #define MAX_BLOCK_SIZE 32
 
 typedef void (*encrypt_func) (void *, nettle_crypt_func, unsigned, uint8_t *,
-			      unsigned, uint8_t *, const uint8_t *);
+                              unsigned, uint8_t *, const uint8_t *);
 typedef void (*decrypt_func) (void *, nettle_crypt_func, unsigned, uint8_t *,
-			      unsigned, uint8_t *, const uint8_t *);
+                              unsigned, uint8_t *, const uint8_t *);
+typedef void (*auth_func) (void *, unsigned, const uint8_t *);
+
+typedef void (*tag_func) (void *, unsigned, uint8_t *);
+
 typedef void (*setkey_func) (void *, unsigned, const uint8_t *);
 
 static void
 stream_encrypt (void *ctx, nettle_crypt_func func, unsigned block_size,
-		uint8_t * iv, unsigned length, uint8_t * dst,
-		const uint8_t * src)
+                uint8_t * iv, unsigned length, uint8_t * dst,
+                const uint8_t * src)
 {
   func (ctx, length, dst, src);
 }
@@ -63,7 +70,7 @@ struct aes_bidi_ctx
 
 static void
 aes_bidi_setkey (struct aes_bidi_ctx *ctx, unsigned length,
-		 const uint8_t * key)
+                 const uint8_t * key)
 {
   aes_set_encrypt_key (&ctx->encrypt, length, key);
   aes_invert_key (&ctx->decrypt, &ctx->encrypt);
@@ -71,14 +78,14 @@ aes_bidi_setkey (struct aes_bidi_ctx *ctx, unsigned length,
 
 static void
 aes_bidi_encrypt (struct aes_bidi_ctx *ctx,
-		  unsigned length, uint8_t * dst, const uint8_t * src)
+                  unsigned length, uint8_t * dst, const uint8_t * src)
 {
   aes_encrypt (&ctx->encrypt, length, dst, src);
 }
 
 static void
 aes_bidi_decrypt (struct aes_bidi_ctx *ctx,
-		  unsigned length, uint8_t * dst, const uint8_t * src)
+                  unsigned length, uint8_t * dst, const uint8_t * src)
 {
   aes_decrypt (&ctx->decrypt, length, dst, src);
 }
@@ -91,7 +98,7 @@ struct camellia_bidi_ctx
 
 static void
 camellia_bidi_setkey (struct camellia_bidi_ctx *ctx, unsigned length,
-		      const uint8_t * key)
+                      const uint8_t * key)
 {
   camellia_set_encrypt_key (&ctx->encrypt, length, key);
   camellia_invert_key (&ctx->decrypt, &ctx->encrypt);
@@ -99,14 +106,14 @@ camellia_bidi_setkey (struct camellia_bidi_ctx *ctx, unsigned length,
 
 static void
 camellia_bidi_encrypt (struct camellia_bidi_ctx *ctx,
-		       unsigned length, uint8_t * dst, const uint8_t * src)
+                       unsigned length, uint8_t * dst, const uint8_t * src)
 {
   camellia_crypt (&ctx->encrypt, length, dst, src);
 }
 
 static void
 camellia_bidi_decrypt (struct camellia_bidi_ctx *ctx,
-		       unsigned length, uint8_t * dst, const uint8_t * src)
+                       unsigned length, uint8_t * dst, const uint8_t * src)
 {
   camellia_crypt (&ctx->decrypt, length, dst, src);
 }
@@ -121,6 +128,9 @@ struct nettle_cipher_ctx
     struct arctwo_ctx arctwo;
     struct des3_ctx des3;
     struct des_ctx des;
+#ifdef NETTLE_GCM
+    struct gcm_aes_ctx aes_gcm;
+#endif
   } ctx;
   void *ctx_ptr;
   uint8_t iv[MAX_BLOCK_SIZE];
@@ -130,9 +140,30 @@ struct nettle_cipher_ctx
   nettle_crypt_func *i_decrypt;
   encrypt_func encrypt;
   decrypt_func decrypt;
+  auth_func auth;
+  tag_func tag;
 };
 
+#define GCM_DEFAULT_NONCE_SIZE 12
 
+#ifdef NETTLE_GCM
+static void _gcm_encrypt(void *_ctx, nettle_crypt_func f,  
+            unsigned block_size, uint8_t *iv,
+            unsigned length, uint8_t *dst,
+            const uint8_t *src)
+{
+  return gcm_aes_encrypt(_ctx, length, dst, src);
+}
+
+static void _gcm_decrypt(void *_ctx, nettle_crypt_func f,  
+            unsigned block_size, uint8_t *iv,
+            unsigned length, uint8_t *dst,
+            const uint8_t *src)
+{
+  return gcm_aes_decrypt(_ctx, length, dst, src);
+}
+
+#endif
 
 static int
 wrap_nettle_cipher_init (gnutls_cipher_algorithm_t algo, void **_ctx)
@@ -150,6 +181,17 @@ wrap_nettle_cipher_init (gnutls_cipher_algorithm_t algo, void **_ctx)
 
   switch (algo)
     {
+#ifdef NETTLE_GCM
+    case GNUTLS_CIPHER_AES_128_GCM:
+      ctx->encrypt = _gcm_encrypt;
+      ctx->decrypt = _gcm_decrypt;
+      ctx->i_encrypt = (nettle_crypt_func*) aes_bidi_encrypt;
+      ctx->auth = (auth_func)gcm_aes_auth;
+      ctx->tag = (tag_func)gcm_aes_digest;
+      ctx->ctx_ptr = &ctx->ctx.aes_gcm;
+      ctx->block_size = AES_BLOCK_SIZE;
+      break;
+#endif
     case GNUTLS_CIPHER_CAMELLIA_128_CBC:
     case GNUTLS_CIPHER_CAMELLIA_256_CBC:
       ctx->encrypt = cbc_encrypt;
@@ -220,6 +262,11 @@ wrap_nettle_cipher_setkey (void *_ctx, const void *key, size_t keysize)
 
   switch (ctx->algo)
     {
+#ifdef NETTLE_GCM
+    case GNUTLS_CIPHER_AES_128_GCM:
+      gcm_aes_set_key(&ctx->ctx.aes_gcm, keysize, key);
+      break;
+#endif
     case GNUTLS_CIPHER_AES_128_CBC:
     case GNUTLS_CIPHER_AES_192_CBC:
     case GNUTLS_CIPHER_AES_256_CBC:
@@ -231,34 +278,34 @@ wrap_nettle_cipher_setkey (void *_ctx, const void *key, size_t keysize)
       break;
     case GNUTLS_CIPHER_3DES_CBC:
       if (keysize != DES3_KEY_SIZE)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INTERNAL_ERROR;
+        }
 
       des_fix_parity (keysize, des_key, key);
 
       /* this fails on weak keys */
       if (des3_set_key (ctx->ctx_ptr, des_key) != 1)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INTERNAL_ERROR;
+        }
       break;
     case GNUTLS_CIPHER_DES_CBC:
       if (keysize != DES_KEY_SIZE)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INTERNAL_ERROR;
+        }
 
       des_fix_parity (keysize, des_key, key);
 
       if (des_set_key (ctx->ctx_ptr, des_key) != 1)
-	{
-	  gnutls_assert ();
-	  return GNUTLS_E_INTERNAL_ERROR;
-	}
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INTERNAL_ERROR;
+        }
       break;
     case GNUTLS_CIPHER_ARCFOUR_128:
     case GNUTLS_CIPHER_ARCFOUR_40:
@@ -278,42 +325,77 @@ wrap_nettle_cipher_setkey (void *_ctx, const void *key, size_t keysize)
 static int
 wrap_nettle_cipher_setiv (void *_ctx, const void *iv, size_t ivsize)
 {
-  struct nettle_cipher_ctx *ctx = _ctx;
+struct nettle_cipher_ctx *ctx = _ctx;
 
-  if (ivsize > ctx->block_size)
+  switch (ctx->algo)
     {
-      gnutls_assert ();
-      return GNUTLS_E_INVALID_REQUEST;
+#ifdef NETTLE_GCM
+    case GNUTLS_CIPHER_AES_128_GCM:
+      if (ivsize != GCM_DEFAULT_NONCE_SIZE)
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
+
+      gcm_aes_set_iv(&ctx->ctx.aes_gcm, GCM_DEFAULT_NONCE_SIZE, iv);
+      break;
+#endif
+    default:
+      if (ivsize > ctx->block_size)
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
+
+      memcpy (ctx->iv, iv, ivsize);
     }
-  memcpy (ctx->iv, iv, ivsize);
 
   return 0;
 }
 
 static int
 wrap_nettle_cipher_decrypt (void *_ctx, const void *encr, size_t encrsize,
-			    void *plain, size_t plainsize)
+                            void *plain, size_t plainsize)
 {
   struct nettle_cipher_ctx *ctx = _ctx;
 
   ctx->decrypt (ctx->ctx_ptr, ctx->i_decrypt, ctx->block_size, ctx->iv,
-		encrsize, plain, encr);
+                encrsize, plain, encr);
 
   return 0;
 }
 
 static int
 wrap_nettle_cipher_encrypt (void *_ctx, const void *plain, size_t plainsize,
-			    void *encr, size_t encrsize)
+                            void *encr, size_t encrsize)
 {
   struct nettle_cipher_ctx *ctx = _ctx;
 
   ctx->encrypt (ctx->ctx_ptr, ctx->i_encrypt, ctx->block_size, ctx->iv,
-		plainsize, encr, plain);
+                plainsize, encr, plain);
 
   return 0;
 }
 
+static int
+wrap_nettle_cipher_auth (void *_ctx, const void *plain, size_t plainsize)
+{
+  struct nettle_cipher_ctx *ctx = _ctx;
+
+  ctx->auth (ctx->ctx_ptr, plainsize, plain);
+
+  return 0;
+}
+
+static void
+wrap_nettle_cipher_tag (void *_ctx, void *tag, size_t tagsize)
+{
+  struct nettle_cipher_ctx *ctx = _ctx;
+
+  ctx->tag (ctx->ctx_ptr, tagsize, tag);
+
+}
+
 static void
 wrap_nettle_cipher_close (void *h)
 {
@@ -322,9 +404,11 @@ wrap_nettle_cipher_close (void *h)
 
 gnutls_crypto_cipher_st _gnutls_cipher_ops = {
   .init = wrap_nettle_cipher_init,
-  .setkey = wrap_nettle_cipher_setkey,
   .setiv = wrap_nettle_cipher_setiv,
+  .setkey = wrap_nettle_cipher_setkey,
   .encrypt = wrap_nettle_cipher_encrypt,
   .decrypt = wrap_nettle_cipher_decrypt,
   .deinit = wrap_nettle_cipher_close,
+  .auth = wrap_nettle_cipher_auth,
+  .tag = wrap_nettle_cipher_tag,
 };
