@@ -1,6 +1,6 @@
 /*
  * GnuTLS PKCS#11 support
- * Copyright (C) 2010,2011 Free Software Foundation
+ * Copyright (C) 2010 Free Software Foundation
  * 
  * Author: Nikos Mavrogiannopoulos
  *
@@ -21,22 +21,21 @@
 */
 
 #include <gnutls_int.h>
+#include <pakchois/pakchois.h>
 #include <gnutls/pkcs11.h>
 #include <stdio.h>
+#include <stdbool.h>
 #include <string.h>
 #include <gnutls_errors.h>
 #include <gnutls_datum.h>
 #include <pkcs11_int.h>
 #include <gnutls_sig.h>
-#include <p11-kit/uri.h>
 
 struct gnutls_pkcs11_privkey_st
 {
   gnutls_pk_algorithm_t pk_algorithm;
   unsigned int flags;
-  struct p11_kit_uri *info;
-  gnutls_pkcs11_pin_callback_t pin_func;
-  void *pin_data;
+  struct pkcs11_url_info info;
 };
 
 /**
@@ -58,14 +57,6 @@ gnutls_pkcs11_privkey_init (gnutls_pkcs11_privkey_t * key)
       return GNUTLS_E_MEMORY_ERROR;
     }
 
-  (*key)->info = p11_kit_uri_new ();
-  if ((*key)->info == NULL)
-    {
-      free (*key);
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
   return 0;
 }
 
@@ -78,13 +69,13 @@ gnutls_pkcs11_privkey_init (gnutls_pkcs11_privkey_t * key)
 void
 gnutls_pkcs11_privkey_deinit (gnutls_pkcs11_privkey_t key)
 {
-  p11_kit_uri_free (key->info);
   gnutls_free (key);
 }
 
 /**
  * gnutls_pkcs11_privkey_get_pk_algorithm:
  * @key: should contain a #gnutls_pkcs11_privkey_t structure
+ * @bits: if bits is non null it will hold the size of the parameters' in bits
  *
  * This function will return the public key algorithm of a private
  * key.
@@ -120,28 +111,23 @@ gnutls_pkcs11_privkey_get_info (gnutls_pkcs11_privkey_t pkey,
                                 gnutls_pkcs11_obj_info_t itype,
                                 void *output, size_t * output_size)
 {
-  return pkcs11_get_info (pkey->info, itype, output, output_size);
+  return pkcs11_get_info (&pkey->info, itype, output, output_size);
 }
 
 
-#define FIND_OBJECT(module, pks, obj, key) \
+#define FIND_OBJECT(pks, obj, key) \
 	do { \
 		int retries = 0; \
 		int rret; \
-		ret = pkcs11_find_object (&module, &pks, &obj, key->info, \
+		ret = pkcs11_find_object (&pks, &obj, &key->info, \
 			SESSION_LOGIN); \
-		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) { \
-			if (token_func) \
-			  { \
-			    rret = pkcs11_call_token_func (key->info, retries++); \
-  			    if (rret == 0) continue; \
-                          } \
+		if (ret < 0) { \
+			rret = token_func(token_data, key->info.token, retries++); \
+			if (rret == 0) continue; \
 			gnutls_assert(); \
 			return ret; \
-		} else if (ret < 0) { \
-                        return ret; \
-                } \
-	} while (0);
+		} \
+	} while (ret < 0);
 
 /*-
  * _gnutls_pkcs11_privkey_sign_hash:
@@ -165,11 +151,10 @@ _gnutls_pkcs11_privkey_sign_hash (gnutls_pkcs11_privkey_t key,
   int ret;
   struct ck_mechanism mech;
   unsigned long siglen;
-  struct ck_function_list *module;
-  ck_session_handle_t pks;
+  pakchois_session_t *pks;
   ck_object_handle_t obj;
 
-  FIND_OBJECT (module, pks, obj, key);
+  FIND_OBJECT (pks, obj, key);
 
   mech.mechanism =
     key->pk_algorithm == GNUTLS_PK_DSA ? CKM_DSA : CKM_RSA_PKCS;
@@ -178,7 +163,7 @@ _gnutls_pkcs11_privkey_sign_hash (gnutls_pkcs11_privkey_t key,
 
   /* Initialize signing operation; using the private key discovered
    * earlier. */
-  rv = pkcs11_sign_init (module, pks, &mech, obj);
+  rv = pakchois_sign_init (pks, &mech, obj);
   if (rv != CKR_OK)
     {
       gnutls_assert ();
@@ -187,7 +172,7 @@ _gnutls_pkcs11_privkey_sign_hash (gnutls_pkcs11_privkey_t key,
     }
 
   /* Work out how long the signature must be: */
-  rv = pkcs11_sign (module, pks, hash->data, hash->size, NULL, &siglen);
+  rv = pakchois_sign (pks, hash->data, hash->size, NULL, &siglen);
   if (rv != CKR_OK)
     {
       gnutls_assert ();
@@ -198,7 +183,7 @@ _gnutls_pkcs11_privkey_sign_hash (gnutls_pkcs11_privkey_t key,
   signature->data = gnutls_malloc (siglen);
   signature->size = siglen;
 
-  rv = pkcs11_sign (module, pks, hash->data, hash->size, signature->data, &siglen);
+  rv = pakchois_sign (pks, hash->data, hash->size, signature->data, &siglen);
   if (rv != CKR_OK)
     {
       gnutls_free (signature->data);
@@ -212,7 +197,7 @@ _gnutls_pkcs11_privkey_sign_hash (gnutls_pkcs11_privkey_t key,
   ret = 0;
 
 cleanup:
-  pkcs11_close_session (module, pks);
+  pakchois_close_session (pks);
 
   return ret;
 }
@@ -236,12 +221,6 @@ gnutls_pkcs11_privkey_import_url (gnutls_pkcs11_privkey_t pkey,
                                   const char *url, unsigned int flags)
 {
   int ret;
-  struct ck_function_list *module;
-  struct ck_attribute *attr;
-  ck_session_handle_t pks;
-  ck_object_handle_t obj;
-  struct ck_attribute a[4];
-  ck_key_type_t key_type;
 
   ret = pkcs11_url_to_info (url, &pkey->info);
   if (ret < 0)
@@ -252,49 +231,19 @@ gnutls_pkcs11_privkey_import_url (gnutls_pkcs11_privkey_t pkey,
 
   pkey->flags = flags;
 
-  attr = p11_kit_uri_get_attribute (pkey->info, CKA_CLASS);
-  if (!attr || attr->value_len != sizeof (ck_object_class_t) ||
-      *(ck_object_class_t*)attr->value != CKO_PRIVATE_KEY)
+  if (pkey->info.type[0] != 0 && strcmp (pkey->info.type, "private") != 0)
     {
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  attr = p11_kit_uri_get_attribute (pkey->info, CKA_ID);
-  if (!attr || !attr->value_len)
+  if (pkey->info.id[0] == 0)
     {
       gnutls_assert ();
       return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
     }
 
-  FIND_OBJECT (module, pks, obj, pkey);
-  a[0].type = CKA_KEY_TYPE;
-  a[0].value = &key_type;
-  a[0].value_len = sizeof (key_type);
-
-  if (pkcs11_get_attribute_value (module, pks, obj, a, 1) == CKR_OK)
-    {
-      switch (key_type)
-        {
-        case CKK_RSA:
-          pkey->pk_algorithm = GNUTLS_PK_RSA;
-          break;
-        case CKK_DSA:
-          pkey->pk_algorithm = GNUTLS_PK_DSA;
-          break;
-        default:
-          _gnutls_debug_log("Cannot determine PKCS #11 key algorithm\n");
-          ret = GNUTLS_E_UNKNOWN_ALGORITHM;
-          goto cleanup;
-        }
-    }
-
-  ret = 0;
-
-cleanup:
-  pkcs11_close_session (module, pks);
-
-  return ret;
+  return 0;
 }
 
 /*-
@@ -320,11 +269,10 @@ _gnutls_pkcs11_privkey_decrypt_data (gnutls_pkcs11_privkey_t key,
   int ret;
   struct ck_mechanism mech;
   unsigned long siglen;
-  struct ck_function_list *module;
-  ck_session_handle_t pks;
+  pakchois_session_t *pks;
   ck_object_handle_t obj;
 
-  FIND_OBJECT (module, pks, obj, key);
+  FIND_OBJECT (pks, obj, key);
 
   mech.mechanism =
     key->pk_algorithm == GNUTLS_PK_DSA ? CKM_DSA : CKM_RSA_PKCS;
@@ -333,7 +281,7 @@ _gnutls_pkcs11_privkey_decrypt_data (gnutls_pkcs11_privkey_t key,
 
   /* Initialize signing operation; using the private key discovered
    * earlier. */
-  rv = pkcs11_decrypt_init (module, pks, &mech, obj);
+  rv = pakchois_decrypt_init (pks, &mech, obj);
   if (rv != CKR_OK)
     {
       gnutls_assert ();
@@ -342,7 +290,7 @@ _gnutls_pkcs11_privkey_decrypt_data (gnutls_pkcs11_privkey_t key,
     }
 
   /* Work out how long the plaintext must be: */
-  rv = pkcs11_decrypt (module, pks, ciphertext->data, ciphertext->size,
+  rv = pakchois_decrypt (pks, ciphertext->data, ciphertext->size,
                          NULL, &siglen);
   if (rv != CKR_OK)
     {
@@ -354,7 +302,7 @@ _gnutls_pkcs11_privkey_decrypt_data (gnutls_pkcs11_privkey_t key,
   plaintext->data = gnutls_malloc (siglen);
   plaintext->size = siglen;
 
-  rv = pkcs11_decrypt (module, pks, ciphertext->data, ciphertext->size,
+  rv = pakchois_decrypt (pks, ciphertext->data, ciphertext->size,
                          plaintext->data, &siglen);
   if (rv != CKR_OK)
     {
@@ -369,7 +317,7 @@ _gnutls_pkcs11_privkey_decrypt_data (gnutls_pkcs11_privkey_t key,
   ret = 0;
 
 cleanup:
-  pkcs11_close_session (module, pks);
+  pakchois_close_session (pks);
 
   return ret;
 }
@@ -392,7 +340,7 @@ gnutls_pkcs11_privkey_export_url (gnutls_pkcs11_privkey_t key,
 {
   int ret;
 
-  ret = pkcs11_info_to_url (key->info, detailed, url);
+  ret = pkcs11_info_to_url (&key->info, detailed, url);
   if (ret < 0)
     {
       gnutls_assert ();
