@@ -29,15 +29,15 @@
 
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
-#include <gnutls_cert.h>
 #include <libtasn1.h>
 #include <gnutls_global.h>
 #include <gnutls_num.h>         /* MAX */
 #include <gnutls_sig.h>
 #include <gnutls_str.h>
 #include <gnutls_datum.h>
-#include "x509_int.h"
+#include <x509_int.h>
 #include <common.h>
+#include <gnutls_pk.h>
 
 static int is_crl_issuer (gnutls_x509_crl_t crl,
                           gnutls_x509_crt_t issuer_cert);
@@ -1194,10 +1194,10 @@ gnutls_x509_crt_verify (gnutls_x509_crt_t cert,
  *   negative error value.
  **/
 int
-gnutls_x509_crl_check_issuer (gnutls_x509_crl_t cert,
+gnutls_x509_crl_check_issuer (gnutls_x509_crl_t crl,
                               gnutls_x509_crt_t issuer)
 {
-  return is_crl_issuer (cert, issuer);
+  return is_crl_issuer (crl, issuer);
 }
 
 /**
