@@ -268,10 +268,6 @@ gnutls_x509_crt_set_crq (gnutls_x509_crt_t crt, gnutls_x509_crq_t crq)
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  result = gnutls_x509_crq_verify(crq, 0);
-  if (result < 0)
-    return gnutls_assert_val(result);
-
   result = asn1_copy_node (crt->cert, "tbsCertificate.subject",
                            crq->crq, "certificationRequestInfo.subject");
   if (result != ASN1_SUCCESS)
@@ -911,7 +907,7 @@ disable_optional_stuff (gnutls_x509_crt_t cert)
 
   if (cert->use_extensions == 0)
     {
-      _gnutls_x509_log ("Disabling X.509 extensions.\n");
+      _gnutls_debug_log ("Disabling X.509 extensions.\n");
       asn1_write_value (cert->cert, "tbsCertificate.extensions", NULL, 0);
     }
 
