@@ -43,7 +43,6 @@ alarm_handler (int signo)
 }
 #else
 #include <windows.h>
-DWORD WINAPI alarm_handler (LPVOID lpParameter);
 DWORD WINAPI
 alarm_handler (LPVOID lpParameter)
 {
@@ -54,7 +53,7 @@ alarm_handler (LPVOID lpParameter)
 }
 
 #define W32_ALARM_VARIABLES HANDLE wtimer = NULL, wthread = NULL; \
-  LARGE_INTEGER alarm_timeout
+  LARGE_INTEGER alarm_timeout = { 0 , 0 }
 #define W32_ALARM_TRIGGER(timeout, leave) { \
   wtimer = CreateWaitableTimer (NULL, TRUE, NULL); \
   if (wtimer == NULL) \
@@ -123,10 +122,11 @@ value2human (double bytes, double time, double *data, double *speed,
 }
 
 static void
-cipher_bench (int algo, int size)
+cipher_mac_bench (int algo, int mac_algo, int size)
 {
   int ret;
   gnutls_cipher_hd_t ctx;
+  gnutls_hmac_hd_t mac_ctx;
   void *_key, *_iv;
   gnutls_datum_t key, iv;
   struct timespec start, stop;
@@ -136,6 +136,7 @@ cipher_bench (int algo, int size)
   int blocksize = gnutls_cipher_get_block_size (algo);
   int keysize = gnutls_cipher_get_key_size (algo);
   char metric[16];
+  int step = size*1024;
 #if defined(_WIN32)
   W32_ALARM_VARIABLES;
 #endif
@@ -156,8 +157,8 @@ cipher_bench (int algo, int size)
   key.data = _key;
   key.size = keysize;
 
-  printf ("Checking %s (%dkb payload)... ", gnutls_cipher_get_name (algo),
-          size);
+  printf ("Checking %s with %s (%dkb payload)... ", gnutls_cipher_get_name (algo),
+      gnutls_mac_get_name(mac_algo), size);
   fflush (stdout);
 
   must_finish = 0;
@@ -169,6 +170,13 @@ cipher_bench (int algo, int size)
 
   gettime (&start);
 
+  ret = gnutls_hmac_init(&mac_ctx, mac_algo, key.data, key.size);
+  if (ret < 0)
+    {
+      fprintf (stderr, "error: %s\n", gnutls_strerror (ret));
+      goto leave;
+    }
+
   ret = gnutls_cipher_init (&ctx, algo, &key, &iv);
   if (ret < 0)
     {
@@ -176,10 +184,95 @@ cipher_bench (int algo, int size)
       goto leave;
     }
 
+  gnutls_hmac(mac_ctx, data, 1024);
+
   do
     {
-      gnutls_cipher_encrypt (ctx, data, size * 1024);
-      data_size += size * 1024;
+      gnutls_hmac(mac_ctx, data, step);
+      gnutls_cipher_encrypt (ctx, data, step);
+      data_size += step;
+    }
+  while (must_finish == 0);
+
+  gnutls_cipher_deinit (ctx);
+  gnutls_hmac_deinit(mac_ctx, NULL);
+
+  gettime (&stop);
+
+  secs = (stop.tv_sec * 1000 + stop.tv_nsec / (1000 * 1000) -
+          (start.tv_sec * 1000 + start.tv_nsec / (1000 * 1000)));
+  secs /= 1000;
+
+  value2human (data_size, secs, &ddata, &dspeed, metric);
+  printf ("Encrypted and hashed %.2f %s in %.2f secs: ", ddata, metric, secs);
+  printf ("%.2f %s/sec\n", dspeed, metric);
+
+leave:
+  free (_key);
+  free (_iv);
+
+}
+
+
+static void
+cipher_bench (int algo, int size, int aead)
+{
+  int ret;
+  gnutls_cipher_hd_t ctx;
+  void *_key, *_iv;
+  gnutls_datum_t key, iv;
+  struct timespec start, stop;
+  double secs;
+  double data_size = 0;
+  double dspeed, ddata;
+  int blocksize = gnutls_cipher_get_block_size (algo);
+  int keysize = gnutls_cipher_get_key_size (algo);
+  char metric[16];
+  int step = size*1024;
+#if defined(_WIN32)
+  W32_ALARM_VARIABLES;
+#endif
+
+  _key = malloc (keysize);
+  if (_key == NULL)
+    return;
+  memset (_key, 0xf0, keysize);
+
+  _iv = malloc (blocksize);
+  if (_iv == NULL)
+    return;
+  memset (_iv, 0xf0, blocksize);
+
+  iv.data = _iv;
+  if (aead) iv.size = 12;
+  else iv.size = blocksize;
+
+  key.data = _key;
+  key.size = keysize;
+
+  printf ("Checking %s (%dkb payload)... ", gnutls_cipher_get_name (algo),
+          size);
+  fflush (stdout);
+
+  must_finish = 0;
+  alarm (5);
+
+  gettime (&start);
+
+  ret = gnutls_cipher_init (&ctx, algo, &key, &iv);
+  if (ret < 0)
+    {
+      fprintf (stderr, "error: %s\n", gnutls_strerror (ret));
+      goto leave;
+    }
+
+  if (aead)
+    gnutls_cipher_add_auth (ctx, data, 1024);
+
+  do
+    {
+      gnutls_cipher_encrypt (ctx, data, step);
+      data_size += step;
     }
   while (must_finish == 0);
 
@@ -213,9 +306,7 @@ mac_bench (int algo, int size)
   double ddata, dspeed;
   int blocksize = gnutls_hmac_get_len (algo);
   char metric[16];
-#if defined(_WIN32)
-  W32_ALARM_VARIABLES;
-#endif
+  int step = size*1024;
 
   _key = malloc (blocksize);
   if (_key == NULL)
@@ -236,8 +327,8 @@ mac_bench (int algo, int size)
 
   do
     {
-      gnutls_hmac_fast (algo, _key, blocksize, data, size * 1024, _key);
-      data_size += size * 1024;
+      gnutls_hmac_fast (algo, _key, blocksize, data, step, _key);
+      data_size += step;
     }
   while (must_finish == 0);
 
@@ -275,25 +366,21 @@ main (int argc, char **argv)
   gnutls_global_set_log_level (debug_level);
   gnutls_global_init ();
 
-  mac_bench (GNUTLS_MAC_SHA1, 4);
-  mac_bench (GNUTLS_MAC_SHA1, 8);
+  gnutls_rnd( GNUTLS_RND_NONCE, data, sizeof(data));
+
+  cipher_bench ( GNUTLS_CIPHER_AES_128_GCM, 16, 1);
+  cipher_mac_bench ( GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA256, 16);
+  cipher_mac_bench ( GNUTLS_CIPHER_AES_128_CBC, GNUTLS_MAC_SHA1, 16);
+
   mac_bench (GNUTLS_MAC_SHA1, 16);
 
-  mac_bench (GNUTLS_MAC_SHA256, 4);
-  mac_bench (GNUTLS_MAC_SHA256, 8);
   mac_bench (GNUTLS_MAC_SHA256, 16);
 
-  cipher_bench (GNUTLS_CIPHER_3DES_CBC, 4);
-  cipher_bench (GNUTLS_CIPHER_3DES_CBC, 8);
-  cipher_bench (GNUTLS_CIPHER_3DES_CBC, 16);
+  cipher_bench (GNUTLS_CIPHER_3DES_CBC, 16, 0);
 
-  cipher_bench (GNUTLS_CIPHER_AES_128_CBC, 4);
-  cipher_bench (GNUTLS_CIPHER_AES_128_CBC, 8);
-  cipher_bench (GNUTLS_CIPHER_AES_128_CBC, 16);
+  cipher_bench (GNUTLS_CIPHER_AES_128_CBC, 16, 0);
 
-  cipher_bench (GNUTLS_CIPHER_ARCFOUR, 4);
-  cipher_bench (GNUTLS_CIPHER_ARCFOUR, 8);
-  cipher_bench (GNUTLS_CIPHER_ARCFOUR, 16);
+  cipher_bench (GNUTLS_CIPHER_ARCFOUR, 16, 0);
 
   return 0;
 }
