@@ -31,6 +31,7 @@ enum
   ACTION_REQUEST,
   ACTION_PUBKEY_INFO,
   ACTION_CERT_PUBKEY,
+  ACTION_VERIFY,
 };
 
 #define TYPE_CRT 1
@@ -55,14 +56,12 @@ typedef struct common_info
   const char *ca_privkey;
 } common_info_st;
 
-gnutls_pubkey_t load_public_key_or_import(int mand, gnutls_privkey_t privkey, common_info_st * info);
-gnutls_privkey_t load_private_key (int mand, common_info_st * info);
-gnutls_x509_privkey_t load_x509_private_key (int mand, common_info_st * info);
+gnutls_x509_privkey_t load_private_key (int mand, common_info_st * info);
 gnutls_x509_crq_t load_request (common_info_st * info);
-gnutls_privkey_t load_ca_private_key (common_info_st * info);
+gnutls_x509_privkey_t load_ca_private_key (common_info_st * info);
 gnutls_x509_crt_t load_ca_cert (common_info_st * info);
 gnutls_x509_crt_t load_cert (int mand, common_info_st * info);
-gnutls_datum *load_secret_key (int mand, common_info_st * info);
+gnutls_datum_t *load_secret_key (int mand, common_info_st * info);
 gnutls_pubkey_t load_pubkey (int mand, common_info_st * info);
 gnutls_x509_crt_t *load_cert_list (int mand, size_t * size,
                                    common_info_st * info);
