@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+ *   2011 Free Software Foundation, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -28,8 +29,6 @@
 #include <gnutls/pkcs11.h>
 #include <gnutls/abstract.h>
 
-#include <gcrypt.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -1318,7 +1317,7 @@ void
 pgp_privkey_info (void)
 {
   gnutls_openpgp_privkey_t key;
-  gnutls_openpgp_keyid_t keyid;
+  unsigned char keyid[GNUTLS_OPENPGP_KEYID_SIZE];
   size_t size;
   int ret, i, subkeys;
   gnutls_datum_t pem;
